--------------------------------------------------------------------------------
-- Touhin!
--

local _, Touhin = ...
LibStub("AceAddon-3.0"):NewAddon(Touhin, "Touhin", "AceEvent-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("Touhin")

-- Debug
_G.Touhin = Touhin

function Touhin:Print(...)
	DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Touhin|r: " .. string.join(" ", tostringall(...)))
end

--------------------------------------------------------------------------------
-- Locals
--

local _G = _G

local media = LibStub("LibSharedMedia-3.0", true)
if media then
	media:Register("statusbar", "TouhinDark", [[Interface\AddOns\Touhin\TouhinDark]])
end
local deformat = LibStub("LibDeformat-3.0").Deformat

local db
local pName = UnitName("player")
local pClass = select(2, UnitClass("player"))
local CLASS_COLORS = {}
local rollTextures = {
	[LOOT_ROLL_TYPE_PASS] = "Interface\\Buttons\\UI-GroupLoot-Pass-Up",
	[LOOT_ROLL_TYPE_NEED] = "Interface\\Buttons\\UI-GroupLoot-Dice-Up",
	[LOOT_ROLL_TYPE_GREED] = "Interface\\Buttons\\UI-GroupLoot-Coin-Up",
	[LOOT_ROLL_TYPE_DISENCHANT] = "Interface\\Buttons\\UI-GroupLoot-DE-Up",
}


--------------------------------------------------------------------------------
-- Anchor
--

local anchor = nil

local function CreateAnchor()
	local f = CreateFrame("Frame", nil, UIParent)
	f:SetWidth(220)
	f:SetHeight(18)
	f:ClearAllPoints()
	f:SetPoint(db.anchor_point or "CENTER", db.anchor_x or 0, db.anchor_y or 0)

	local texture = f:CreateTexture(nil, "BACKGROUND")
	texture:SetAllPoints(f)
	texture:SetBlendMode("BLEND")
	texture:SetTexture(0, 0, 0, 0.3)

	local text = f:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
	text:SetAllPoints(f)
	text:SetText("Touhin")

	f:EnableMouse(true)
	f:SetClampedToScreen(true)
	f:SetScript("OnMouseUp", function(self, button)
		if button == "RightButton" then
			local point, _, _, x, y = self:GetPoint(1)
			db.anchor_point = point
			db.anchor_x = x
			db.anchor_y = y
			self:Hide()
		elseif button == "LeftButton" and IsShiftKeyDown() then
			Touhin:AddLootTest()
		elseif button == "MiddleButton" then
			Touhin:OpenOptions()
		end
	end)

	f:SetMovable(true)
	f:RegisterForDrag("LeftButton")
	f:SetScript("OnDragStart", f.StartMoving)
	f:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing()
		local point, _, _, x, y = self:GetPoint(1)
		db.anchor_point = point
		db.anchor_x = x
		db.anchor_y = y
	end)

	f:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
		GameTooltip:ClearLines()
		GameTooltip:AddLine("Touhin")
		GameTooltip:AddLine(L.ANCHOR_TOOLTIP, 0.2, 1, 0.2, 1)
		GameTooltip:Show()
	end)
	f:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)

	f:Hide()

	anchor = f
end

local function ShowAnchor()
	if not anchor then CreateAnchor() end
	anchor:Show()
end

local function HideAnchor()
	if not anchor then return end
	anchor:Hide()
end

local function UpdateAnchor()
	if not anchor then return end
	anchor:ClearAllPoints()
	anchor:SetPoint(db.anchor_point or "CENTER", db.anchor_x or 0, db.anchor_y or 0)
end

local function ResetAnchor()
	if anchor then
		anchor:ClearAllPoints()
		anchor:SetPoint("CENTER")
	end
	db.anchor_point = nil
	db.anchor_x = nil
	db.anchor_y = nil

	ShowAnchor()
end

local function ToggleAnchor()
	if not anchor or not anchor:IsShown() then
		ShowAnchor()
	else
		HideAnchor()
	end
end


--------------------------------------------------------------------------------
-- Frame stacking
--

local UpdateRows, AddRow, RemoveRow, GetRows, UpdateRowOrdering
do
	local function sorter(a, b)
		return a.time < b.time and true or false
	end

	local rows = {}
	local tmp = {}

	local point, relativePoint, yOffset
	function UpdateRowOrdering()
		point = (db.growDown and "TOP" or "BOTTOM") .. (db.alignLeft and "LEFT" or "RIGHT")
		relativePoint = (db.growDown and "BOTTOM" or "TOP") .. (db.alignLeft and "LEFT" or "RIGHT")
		yOffset = db.rowSpacing * (db.growDown and -1 or 1)
	end

	-- FIXME the text seems to "bounce" up one slot when arranging ;[
	function UpdateRows()
		wipe(tmp)
		for row in next, rows do
			row:Hide()
			tmp[#tmp + 1] = row
		end
		table.sort(tmp, sorter)

		if not point then UpdateRowOrdering() end

		local last = anchor
		for i=1, db.shownLimit do
			if not tmp[i] then break end
			tmp[i]:ClearAllPoints()
			tmp[i]:SetScale(db.scale)
			tmp[i]:SetPoint(point, last, relativePoint, 0, yOffset)
			tmp[i]:Show()
			last = tmp[i]
		end
	end

	function AddRow(row)
		if not anchor then CreateAnchor() end
		rows[row] = true
		UpdateRows()
	end

	function RemoveRow(row)
		rows[row] = nil
		row:Hide()
		UpdateRows()
	end

	function GetRows()
		return rows
	end
end


--------------------------------------------------------------------------------
-- Frame creation
--

local GetFrame, ReleaseFrame, UpdateFrameBackdrop, UpdateFrameFont
do
	local frameHeap = {}

	--scripts
	local function fader_OnFinished(self, requested)
		RemoveRow(self.object)
		ReleaseFrame(self.object)
	end

	local function frame_OnShow(self)
		if db.fadeDelay > 0 then
			self.fader:GetParent():Play()
		end
	end

	local function frame_OnEnter(self)
		if not self.link then return end
		GameTooltip:SetOwner(self, "ANCHOR_" .. (db.alignLeft and "RIGHT" or "LEFT"), 0, -self:GetHeight())
		GameTooltip:SetHyperlink(self.link)

		if self.rollId then
			GameTooltip:AddLine(" ")
			GameTooltip:AddLine(_G.LOOT_ROLLS, _G.NORMAL_FONT_COLOR.r, _G.NORMAL_FONT_COLOR.g, _G.NORMAL_FONT_COLOR.b)
			GameTooltip:AddTexture([[Interface\Minimap\Tracking\Banker]])
			for itemIdx=1, C_LootHistory.GetNumItems() do
				local rollId, _, numPlayers, _, winnerIdx, isMasterLoot = C_LootHistory.GetItem(itemIdx)
				if self.rollId == rollId then
					if not isDone or winnerIdx or isMasterLoot then
						for playerIdx=1, numPlayers do
							local name, class, rollType, roll, isWinner, isMe = C_LootHistory.GetPlayerInfo(itemIdx, playerIdx)
							if isMe or roll or isMasterLoot then --should show
								local line = string.format("%s%s|r", CLASS_COLORS[class], name)
								if roll and roll > 0 then
									line = line .. " - " .. roll
								end
								GameTooltip:AddLine(line, 1, 1, 1)
								GameTooltip:AddTexture(rollTextures[rollType])
							end
						end
					else --everyone passed
						GameTooltip:AddLine(_G.LOOT_HISTORY_ALL_PASSED, 1, 1, 1)
						GameTooltip:AddTexture(rollTextures[_G.LOOT_ROLL_TYPE_PASS])
					end
					break
				end
			end
			GameTooltip:Show()
		end

		if IsModifiedClick("COMPAREITEMS") or (GetCVarBool("alwaysCompareItems") and not GameTooltip:IsEquippedItem()) then
			GameTooltip_ShowCompareItem()
		end
		if IsModifiedClick("DRESSUP") then
			ShowInspectCursor()
		else
			ResetCursor()
		end

		if db.fadeDelay > 0 then
			self.fader:GetParent():Pause()
		end
	end

	local function frame_OnLeave(self)
		if not self.link then return end
		GameTooltip:Hide()
		ResetCursor()
		frame_OnShow(self)
	end

	local function frame_OnClick(self, button)
		if button == "LeftButton" then
			if IsModifiedClick() then
				HandleModifiedItemClick(self.link)
			elseif self.rollId then
				LootHistoryFrame_OpenToRoll(LootHistoryFrame, self.rollId)
			end
		elseif button == "RightButton" then
			if db.fadeDelay > 0 then
				self.fader:GetParent():Stop()
			else
				fader_OnFinished(self.fader, true)
			end
		end
	end


	--media
	local font = _G.STANDARD_TEXT_FONT
	local background = "Interface\\AddOns\\Touhin\\TouhinDark"
	local backdrop = {
		bgFile = "Interface\\Tooltips\\gloss",
		tile = true, tileSize = 16,
		edgeFile = "Interface\\Tooltips\\gloss", edgeSize = 16,
		insets = {left = 4, right = 4, top = 4, bottom = 4},
	}

	function UpdateFrameBackdrop()
		if not media then return end

		local bgFile, edgeFile, edgeSize, inset = db.bgFile, db.edgeFile, db.edgeSize, db.insets

		if media:Fetch("statusbar", bgFile, true) then
			background = media:Fetch("statusbar", bgFile)
		end
		if media:Fetch("border", edgeFile, true) then
			backdrop.edgeFile = media:Fetch("border", edgeFile)
		end
		if edgeSize then
			backdrop.edgeSize = edgeSize
		end
		if inset then
			backdrop.insets.left = inset
			backdrop.insets.right = inset
			backdrop.insets.top = inset
			backdrop.insets.bottom = inset
		end

		--XXX border glitches occasionally on reused frames, clean our heap if the backdrop changes
		for k in next, GetRows() do
			k.dirty = true
		end
		for i = 1, #frameHeap do
			frameHeap[i]:SetParent(nil)
			frameHeap[i] = nil
		end
	end

	function UpdateFrameFont()
		if not media then return end

		local fontName = db.font
		if media:Fetch("font", fontName, true) then
			font = media:Fetch("font", fontName)
		end
	end


	--prototype
	local function SetRow(self, icon, color, text, hightlightColor)
		self.text:SetText(text)

		if not db.colorBackground then
			color = db.bgColor
		end
		local borderColor = hightlightColor or color

		self:SetBackdropBorderColor(borderColor.r, borderColor.g, borderColor.b)
		self.background:SetVertexColor(color.r, color.g, color.b, .7)
		self.iconFrame:SetBackdropBorderColor(color.r, color.g, color.b)
		self.icon:SetTexture(icon)

		-- make the frame fit the text (+4 is just some arbitrary padding)
		local width = db.insets + db.insets + 4 + db.iconGap

		if db.showIcon then
			width = width + self.iconFrame:GetWidth()

			if self.rollType then
				self.rollIcon:SetNormalTexture(rollTextures[self.rollType])
				self.rollIcon:Show()
			end
		end

		local textWidth = self.text:GetStringWidth()
		self.text:SetWidth(textWidth)
		width = width + textWidth

		self:SetWidth(width)
	end


	--creation/recycling
	function GetFrame()
		local frame = tremove(frameHeap)
		--print("GetFrame", frame)
		if not frame then
			--shiney new frame
			frame = CreateFrame("Frame", nil, UIParent)
			--frame:SetWidth(300)
			frame:SetHeight(22)
			frame:SetBackdrop(backdrop)
			frame:SetBackdropColor(db.bgColor.r, db.bgColor.g, db.bgColor.b, db.bgColor.a)
			frame:SetFrameStrata(db.strata)

			local texture = frame:CreateTexture(nil, "BORDER")
			texture:SetPoint("TOPLEFT", frame, "TOPLEFT", db.insets, -db.insets)
			texture:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -db.insets, db.insets)
			texture:SetTexture(background)
			frame.background = texture

			local iconFrame = CreateFrame("Frame", nil, frame)
			iconFrame:SetPoint("LEFT", db.insets, 0)
			iconFrame:SetSize(db.iconSize, db.iconSize)
			iconFrame:SetBackdrop(backdrop)
			iconFrame:SetBackdropColor(1, 1, 1, 0)
			frame.iconFrame = iconFrame

			local icon = iconFrame:CreateTexture(nil, "OVERLAY")
			icon:SetPoint("TOPLEFT", iconFrame, "TOPLEFT", db.insets, -db.insets)
			icon:SetPoint("BOTTOMRIGHT", iconFrame, "BOTTOMRIGHT", -db.insets, db.insets)
			icon:SetTexCoord(.07, .93, .07, .93) --0.1, 0.9, 0.1, 0.9
			frame.icon = icon

			local rollIcon = CreateFrame("Button", nil, iconFrame)
			rollIcon:SetPoint("BOTTOMRIGHT")
			rollIcon:SetSize(db.iconSize, db.iconSize)
			rollIcon:SetScale(db.rollIconScale)
			rollIcon:Hide()
			frame.rollIcon = rollIcon

			local text = frame:CreateFontString(nil, "OVERLAY")
			text:SetPoint("LEFT", iconFrame, "RIGHT", db.iconGap, 0)
			text:SetFont(font, db.fontSize)
			text:SetJustifyH("LEFT")
			text:SetJustifyV("MIDDLE")
			text:SetTextColor(1, 1, 1, 1)
			text:SetShadowOffset(0.8, -0.8)
			text:SetShadowColor(0, 0, 0, 1)
			frame.text = text

			--a timer and fancy fading in one, pretty neat
			local ag = frame:CreateAnimationGroup()
			ag:SetLooping("NONE")

			local fader = ag:CreateAnimation("Alpha")
			fader:SetChange(db.fadeTime > 0 and -1 or 1)
			fader:SetStartDelay(db.fadeDelay)
			fader:SetDuration(db.fadeTime > 0 and db.fadeTime or 0.1)
			fader:SetSmoothing("IN")
			fader:SetScript("OnStop", fader_OnFinished)
			fader:SetScript("OnFinished", fader_OnFinished)
			fader.object = frame
			frame.fader = fader

			--the FontString width is shorter by like half a space if the frame isn't shown atleast once. weird, eh?
			frame:Show()
			frame:Hide()

			frame:SetScript("OnShow", frame_OnShow)
			frame:SetScript("OnEnter", frame_OnEnter)
			frame:SetScript("OnLeave", frame_OnLeave) --GameTooltip_HideResetCursor
			frame:SetScript("OnMouseUp", frame_OnClick)

			frame.SetRow = SetRow
		else
			frame:SetBackdropColor(db.bgColor.r, db.bgColor.g, db.bgColor.b, db.bgColor.a)
			frame:SetFrameStrata(db.strata)
			frame.background:SetVertexColor(1, 1, 1, 1)

			frame.iconFrame:SetBackdropColor(1, 1, 1, 0)
			frame.iconFrame:SetSize(db.iconSize, db.iconSize)
			frame.iconFrame:SetPoint("LEFT", db.insets, 0)
			frame.iconFrame:Show()

			frame.icon:SetPoint("TOPLEFT", frame.iconFrame, "TOPLEFT", db.insets, -db.insets)
			frame.icon:SetPoint("BOTTOMRIGHT", frame.iconFrame, "BOTTOMRIGHT", -db.insets, db.insets)

			frame.rollIcon:SetSize(db.iconSize, db.iconSize)
			frame.rollIcon:SetScale(db.rollIconScale)
			frame.rollIcon:Hide()

			frame.text:SetPoint("LEFT", frame.iconFrame, "RIGHT", db.iconGap, 0)
			frame.text:SetFont(font, db.fontSize)

			frame.fader:SetStartDelay(db.fadeDelay)
			frame.fader:SetChange(db.fadeTime > 0 and -1 or 1)
			frame.fader:SetDuration(db.fadeTime > 0 and db.fadeTime or 0.1)

			frame.item = nil
			frame.amount = nil
			frame.link = nil
			frame.unit = nil
			frame.time = nil
			frame.rollId = nil
			frame.rollType = nil
		end

		if not db.showIcon then
			frame.text:SetPoint("LEFT", frame.iconFrame, db.iconGap, 0)
			frame.iconFrame:Hide()
		end

		return frame
	end

	function ReleaseFrame(frame)
		--print("ReleaseFrame", frame.fader:IsStopped() or frame.fader:IsDone())
		frame:Hide()
		frame:ClearAllPoints()

		if frame.dirty then
			frame:SetParent(nil) --YOU ARE DISOWNED!
			return
		end

		tinsert(frameHeap, frame)
	end
end


--------------------------------------------------------------------------------
-- Parse chat messages
--

--[[
CURRENCY_GAINED = "You receive currency: %s.";
CURRENCY_GAINED_MULTIPLE = "You receive currency: %s x%d.";
--]]

local function ParseCurrencyMessage(msg)
	local item, quantity = deformat(msg, _G.CURRENCY_GAINED_MULTIPLE)
	if quantity then
		return item, tonumber(quantity)
	end

	item = deformat(msg, _G.CURRENCY_GAINED)
	if item then
		return item, 1
	end
end

--/run Touhin:CHAT_MSG_CURRENCY(nil, [[You receive currency: |cff00aa00|Hcurrency:416|h[Mark of the World Tree]|h|r x1.]])
function Touhin:CHAT_MSG_CURRENCY(_, msg)
	if db.showAltCurrency then
		item, quantity = ParseCurrencyMessage(msg)
		if quantity then
			self:AddCurrency(item, quantity)
			return
		end
	end
end

--[[
LOOT_ITEM = "%s receives loot: %s.";
LOOT_ITEM_MULTIPLE = "%s receives loot: %sx%d.";
LOOT_ITEM_PUSHED_SELF = "You receive item: %s.";
LOOT_ITEM_PUSHED_SELF_MULTIPLE = "You receive item: %sx%d.";
LOOT_ITEM_SELF = "You receive loot: %s.";
LOOT_ITEM_SELF_MULTIPLE = "You receive loot: %sx%d.";
LOOT_ITEM_CREATED_SELF = "You create: %s.";
LOOT_ITEM_CREATED_SELF_MULTIPLE = "You create: %sx%d.";
LOOT_DISENCHANT_CREDIT = "%s was disenchanted for loot by %s.";

LOOT_ROLL_ALL_PASSED = "|HlootHistory:%d|h[Loot]|h: Everyone passed on: %s";
--]]
local function ParseLootMessage(msg)
	local player = pName
	local item, quantity = deformat(msg, _G.LOOT_ITEM_SELF_MULTIPLE)
	if quantity then
		return player, item, tonumber(quantity)
	end

	item = deformat(msg, _G.LOOT_ITEM_SELF)
	if item then
		return player, item, 1
	end

	local item, quantity = deformat(msg, _G.LOOT_ITEM_PUSHED_SELF_MULTIPLE)
	if quantity then
		return player, item, tonumber(quantity)
	end

	item = deformat(msg, _G.LOOT_ITEM_PUSHED_SELF)
	if item then
		return player, item, 1
	end

	--created is kind of wonky with totals, special case it to never show totals! ;[
	item, quantity = deformat(msg, _G.LOOT_ITEM_CREATED_SELF_MULTIPLE)
	if quantity then
		return player, item, tonumber(quantity), 1
	end

	item = deformat(msg, _G.LOOT_ITEM_CREATED_SELF)
	if item then
		return player, item, 1, 1
	end

	player, item, quantity = deformat(msg, _G.LOOT_ITEM_MULTIPLE)
	if quantity then
		return player, item, tonumber(quantity)
	end

	player, item = deformat(msg, _G.LOOT_ITEM)
	if item then
		return player, item, 1
	end

	item = select(2, deformat(msg, _G.LOOT_ROLL_ALL_PASSED))
	if item then
		return L["Nobody"], item, 0
	end
end

local ParseRollMessage, GetItemRoll
do
	--[[
	probably more efficient to use the LOOT_ROLL events, but i'm not sure about the order. The loot won/received CHAT_MSG's are back to back

	LOOT_ROLL_YOU_WON_NO_SPAM_DE = "|HlootHistory:%d|h[Loot]|h: You (Disenchant - %d) Won: %s";
	LOOT_ROLL_YOU_WON_NO_SPAM_GREED = "|HlootHistory:%d|h[Loot]|h: You (Greed - %d) Won: %s";
	LOOT_ROLL_YOU_WON_NO_SPAM_NEED = "|HlootHistory:%d|h[Loot]|h: You (Need - %d) Won: %s";

	LOOT_ROLL_WON_NO_SPAM_DE = "|HlootHistory:%d|h[Loot]|h: %s (Disenchant - %d) Won: %s";
	LOOT_ROLL_WON_NO_SPAM_GREED = "|HlootHistory:%d|h[Loot]|h: %s (Greed - %d) Won: %s";
	LOOT_ROLL_WON_NO_SPAM_NEED = "|HlootHistory:%d|h[Loot]|h: %s (Need - %d) Won: %s";
	--]]
	local rolls = {}
	local strings = {
		_G.LOOT_ROLL_WON_NO_SPAM_DE,
		_G.LOOT_ROLL_WON_NO_SPAM_GREED,
		_G.LOOT_ROLL_WON_NO_SPAM_NEED,
	}

	function ParseRollMessage(msg)
		--if not IsInGroup() then return end
		for _,rollString in ipairs(strings) do
			local rollId, player, roll, item = deformat(msg, rollString)
			if item then
				rolls[item] = rollId
				return true
			end
		end
	end

	local L_ENCHANTING = select(7, GetAuctionItemSubClasses(6))
	function GetItemRoll(item, name)
		local rollId = rolls[item]
		if not rollId and select(7, GetItemInfo(item)) ~= L_ENCHANTING then return end --no roll info, not enchanting mats
		rolls[item] = nil

		for i=1, C_LootHistory.GetNumItems() do
			local rId, itemLink, _, _, winnerIndex = C_LootHistory.GetItem(i)
			if winnerIndex then
				local winner, _, rollType = C_LootHistory.GetPlayerInfo(i, winnerIndex)
				if rId == rollId then
					return rollType, rId
				elseif not rollId and rollType == 3 and name == winner:gsub("%-.+", "") and rId == rolls[itemLink] then
					--no rollId (de result) check if we can match a rollId to the actual item
					rolls[itemLink] = nil
					return rollType, rId, itemLink
				end
			end
		end
	end
end

--/spew GetItemInfo(88296)
--/run Touhin:CHAT_MSG_LOOT(nil, LOOT_ROLL_WON_NO_SPAM_NEED:format(2, "Neb", 100, select(2, GetItemInfo(88296))))
--/run Touhin:CHAT_MSG_LOOT(nil, LOOT_ITEM:format("Neb", select(2, GetItemInfo(88296))))
function Touhin:CHAT_MSG_LOOT(_, msg)
	if ParseRollMessage(msg) then return end --need/greed/de won but not looted yet, save the item link and roll id

	local player, item, quantity, special = ParseLootMessage(msg)
	if item then
		self:AddLoot(player, item, quantity, special)
		return
	end
end


--[[
LOOT_MONEY = "%s loots %s.";x
LOOT_MONEY_SPLIT = "Your share of the loot is %s.";
LOOT_MONEY_SPLIT_GUILD = "Your share of the loot is %s. (%s deposited to guild bank)";
YOU_LOOT_MONEY = "You loot %s";
YOU_LOOT_MONEY_GUILD = "You loot %s (%s deposited to guild bank)";
--]]
local ParseMoneyMessage
do
	-- Parse coin strings (code from SpecialEventsEmbed-Loot)
	local cg = _G.GOLD_AMOUNT:gsub("%%d", "(%1+)")
	local cs = _G.SILVER_AMOUNT:gsub("%%d", "(%1+)")
	local cc = _G.COPPER_AMOUNT:gsub("%%d", "(%1+)")

	local function parseCoinString(str)
		local g = string.match(str, cg) or 0
		local s = string.match(str, cs) or 0
		local c = string.match(str, cc) or 0
		return g * 10000 + s * 100 + c
	end

	local strings = {
		_G.LOOT_MONEY_SPLIT,
		_G.LOOT_MONEY_SPLIT_GUILD,
		_G.YOU_LOOT_MONEY,
		_G.YOU_LOOT_MONEY_GUILD,
	}

	function ParseMoneyMessage(msg)
		for _,moneyString in ipairs(strings) do
			local money = deformat(msg, moneyString)
			if money then
				return pName, parseCoinString(money)
			end
		end

		local player, money = deformat(msg, _G.LOOT_MONEY)
		if money then
			return player, parseCoinString(money)
		end
	end
end

function Touhin:CHAT_MSG_MONEY(_, msg)
	if db.showMoney then
		local player, quantity = ParseMoneyMessage(msg)
		if quantity then
			self:AddCoin(quantity)
			return
		end
	end
end


--------------------------------------------------------------------------------
-- Show Loot!
--

function Touhin:AddLootTest()
	if db.showMoney and math.random() > .8 then
		self:AddCoin(math.random(1, 50000))
	else
		local item
		repeat
			item = GetContainerItemLink(math.random(0, 4), math.random(1, 22))
		until item
		self:AddLoot(pName, item, 0)
	end
end

do
	local function getCoinString(amount)
		local s = GetCoinText(amount, " ")
		s = string.gsub(s, "(%d+) Gold", "|cffffffff%1|r|cffffd700g|r")
		s = string.gsub(s, "(%d+) Silver", "|cffffffff%1|r|cffc7c7cfs|r")
		s = string.gsub(s, "(%d+) Copper", "|cffffffff%1|r|cffeda55fc|r")
		return s
	end

	function Touhin:AddCoin(amount)
		local row
		for k in next, GetRows() do
			if k.item == "coin" then
				if not k.dirty then
					row = k
				else
					--XXX update for the proper text width when toggling the icon
					RemoveRow(k)
					ReleaseFrame(k)
				end
				break
			end
		end
		if not row then
			row = GetFrame()
			row.time = 0 --always on top (giggity)
			row.item = "coin"
		end

		local text
		if row.amount then
			row.amount = row.amount + amount
			text = string.format("%s (|cff20ff20+|r%s)", getCoinString(row.amount), getCoinString(amount))
		else
			row.amount = amount
			text = getCoinString(amount)
		end

		row:SetRow(GetCoinIcon(row.amount), _G.ITEM_QUALITY_COLORS[0], text)
		AddRow(row)
	end
end

do
	local currencyIds = {
		[61] = true,  -- Dalaran Jewelcrafting Token
		[81] = true,  -- Epicurean's Award
		[241] = true, -- Champion's Seal
		[361] = true, -- Illustrious Jewelcrafter's Token
		[390] = true, -- Conquest Points (CONQUEST_CURRENCY)
		[391] = true, -- Tol Barad Commendation
		--[392] = true, -- Honor Points (HONOR_CURRENCY)
		[393] = true, -- Fossil Fragment
		[394] = true, -- Night Elf Fragment
		[395] = true, -- Justice Points (JUSTICE_CURRENCY)
		[396] = true, -- Valor Points (VALOR_CURRENCY)
		[397] = true, -- Orc Fragment
		[398] = true, -- Draenei Fragment
		[399] = true, -- Vrykul Fragment
		[400] = true, -- Nerubian Fragment
		[401] = true, -- Tol'vir Fragment
		[402] = true, -- Ironpaw Token Chef's Award
		[416] = true, -- Mark of the World Tree
		[515] = true, -- Darkmoon Prize Ticket
		[614] = true, -- Mote of Darkness
		[615] = true, -- Essence of Corrupted Deathwing
		[676] = true, -- Pandaren Archaeology Fragment
		[677] = true, -- Mogu Archaeology Fragment
		[697] = true, -- Elder Charm of Good Fortune
		[698] = true, -- Zen Jewelcrafter's Token
	}

	function Touhin:AddCurrency(item, amount)
		--item is a link!
		local id = string.match(item, "|Hcurrency:(%d+)|h%[(.+)%]|h")
		if not id or not currencyIds[tonumber(id)] then return end

		local name, count, texture = GetCurrencyInfo(id)
		local classColor = CLASS_COLORS[pClass]

		local text = string.format("%s%s|r |cffffffff[%s]|r |cffaaaaaa%s|r", classColor, pName, name, db.showTotalCount and count or amount)

		local row = GetFrame()
		row.unit = pName
		row.time = GetTime()
		row.link = item

		row:SetRow("Interface\\Icons\\"..texture, _G.ITEM_QUALITY_COLORS[0], text)
		AddRow(row)
	end
end

do
	local L_QUEST = select(10, GetAuctionItemClasses())
	local QUEST_COLOR = { hex = "|cffffff00", r = 1, g = 1, b = 0 } --YELLOW_FONT_COLOR + YELLOW_FONT_COLOR_CODE

	function Touhin:AddLoot(unitName, item, quantity, isCrafted)
		--print("AddLoot", unitName, item, quantity)
		local rollType, rollId, deItem
		if not isCrafted then
			rollType, rollId, deItem = GetItemRoll(item, unitName)
			if deItem then
				quantity = 1
			end
		end

		local itemName, itemLink, itemQuality, _, _, itemType, itemSubType, _, _, itemTexture = GetItemInfo(deItem or item)
		if not itemName then return end

		local isQuestItem = itemType == L_QUEST
		local isMine = UnitIsUnit("player", unitName)

		--threshold tests
		if not (isQuestItem and db.showQuestItems) and ( (isMine and itemQuality < db.qualitySelfThreshold) or (not isMine and itemQuality < db.qualityThreshold) ) then
			return
		end

		local totalText = ""
		if isMine and not isCrafted and db.showTotalCount then
			local total = quantity + GetItemCount(itemLink) or 0
			if total > quantity and total > 1 then --extra check for test loot
				totalText = string.format(" |cffaaaaaa%d|r", total)
			end
		end

		if unitName == L["Nobody"] then
			itemTexture = rollTextures[_G.LOOT_ROLL_TYPE_PASS]
		end

		--TODO should probably move the item name to its own fontstring and make the option control row width, but ugh.
		if string.len(itemName) > db.displayLength then
			itemName = string.sub(itemName, 1, db.displayLength) .. ".."
		end
		local classColor = CLASS_COLORS[select(2, UnitClass(unitName)) or true] or "|cffcccccc"
		local qualityColor = isQuestItem and QUEST_COLOR or _G.ITEM_QUALITY_COLORS[itemQuality]

		local rowText
		if quantity > 1 then
			rowText = string.format("%s%s|r %s[%s]|r|cffffffffx%d|r%s", classColor, unitName, qualityColor.hex, itemName, quantity, totalText)
		else
			rowText = string.format("%s%s|r %s[%s]|r%s", classColor, unitName, qualityColor.hex, itemName, totalText)
		end

		local highlight = db.highlightWon and isMine and rollType and QUEST_COLOR
		if highlight then
			PlaySoundKitID(31578)
		end

		local row = GetFrame()
		row.link = itemLink
		row.unit = unitName
		row.time = GetTime()

		if rollType then
			row.rollId = rollId
			row.rollType = rollType
		end

		row:SetRow(itemTexture, qualityColor, rowText, highlight)
		AddRow(row)
	end
end


--------------------------------------------------------------------------------
-- Options
--

local defaults = {
	profile = {
		scale = 1,
		showMoney = true,
		showAltCurrency = true,
		showLoot = true,
		showTotalCount = true,
		showRollWinner = true,
		showRollChoices = true,
		showQuestItems = true,
		highlightWon = true,
		disableLootAlert = false,
		qualityThreshold = 3,
		qualitySelfThreshold = 0,
		displayLength = 100,
		growDown = true,
		alignLeft = true,
		shownLimit = 10,
		fadeDelay = 8,
		fadeTime = .4,
		strata = "LOW",
		font = "Arial",
		fontSize = 12,
		bgFile = "TouhinDark",
		edgeFile = "Blizzard Tooltip",
		edgeSize = 16,
		insets = 4,
		iconGap = 2,
		rowSpacing = 2,
		bgColor = { r = 0, b = 0, g = 0, a = .9 },
		borderAlpha = 1,
		colorBackground =  true,
		showIcon = true,
		iconSize = 26,
		rollIconScale = .6,
	},
}

local GetOptions
do
	local qualityValues = {}
	for i=0, 5 do qualityValues[i] = format("%s%s|r", _G["ITEM_QUALITY_COLORS"][i].hex, _G["ITEM_QUALITY"..i.."_DESC"]) end
	local strataValues = {[1] = "BACKGROUND", [2] = "LOW", [3] = "MEDIUM", [4] = "HIGH", [5] = "DIALOG", [6] = "TOOLTIP"}

	function GetOptions()
		local options = {
			name = "Touhin",
			type = "group",
			get = function(info) return db[info[#info]] end,
			set = function(info, value) db[info[#info]] = value end,
			args = {
				header = {
					name = GetAddOnMetadata("Touhin", "Notes").."\n",
					type = "description",
					order = 1,
					width = "full",
				},
				toggleAnchor = {
					name = L["Toggle Anchor"],
					desc = L["Toggle showing the anchor."],
					type = "execute",
					func = ToggleAnchor,
					order = 2,
				},
				resetAnchor = {
					name = L["Reset Position"],
					desc = L["Reset the anchor position to the center of your screen."],
					type = "execute",
					func = ResetAnchor,
					order = 3,
				},
				growDown = {
					name = L["Grow Down"],
					desc = L["New items are added below shown items instead of above."],
					type = "toggle",
					set = function(info, value)
						db[info[#info]] = value
						UpdateRowOrdering()
						UpdateRows()
					end,
					order = 4,
				},
				alignLeft = {
					name = L["Align Left"],
					desc = L["Items are aligned to the left edge of the anchor instead of the right."],
					type = "toggle",
					set = function(info, value)
						db[info[#info]] = value
						UpdateRowOrdering()
						UpdateRows()
					end,
					order = 5,
				},
				scale = {
					name = L["Scale"],
					desc = L["Set the overall scale of the display."],
					type = "range", isPercent = true, min = 0, softMin = .5, softMax = 1.5, max = 3, step = .01,
					set = function(info, value)
						db[info[#info]] = value
						UpdateRows()
					end,
					order = 6,
				},
				shownLimit = {
					name = L["Shown Limit"],
					desc = L["Maximum number of items to show at once."],
					type = "range", min = 1, max = 25, step = 1,
					set = function(info, value)
						db[info[#info]] = value
						UpdateRows()
					end,
					order = 7,
				},
				showMoney = {
					name = L["Show Money"],
					desc = L["Toggle showing looted money received by you in the display."],
					type = "toggle",
					order = 10,
				},
				showAltCurrency = {
					name = L["Show Currency"],
					desc = L["Toggle showing currency (tokens, points, etc.) received by you in the display."],
					type = "toggle",
					order = 11,
				},
				showTotalCount = {
					name = L["Show Item Total"],
					desc = L["Toggle showing the total number of items in your inventory."],
					type = "toggle",
					order = 12,
				},
				showQuestItems = {
					name = L["Show Quest Items"],
					desc = L["Ignore loot thresholds and always show looted quest items."],
					type = "toggle",
					order = 13,
				},
				highlightWon = {
					name = L["Highlight Won"],
					desc = L["Toggle highlighting items won while in a group."],
					type = "toggle",
					order = 13.1,
				},
				disableLootAlert = {
					name = L["Hide Loot Alert"],
					desc = L["Disable showing the loot won frame."],
					type = "toggle",
					set = function(info, value)
						db[info[#info]] = value
						AlertFrame:UnregisterEvent("LOOT_ITEM_ROLL_WON")
						if not value then
							AlertFrame:RegisterEvent("LOOT_ITEM_ROLL_WON")
						end
					end,
					order = 13.2,
				},
				qualitySelfThreshold = {
					name = L["Self Quality Threshold"],
					desc = L["Set the minimum quality of items looted by you to show."],
					type = "select",
					values = qualityValues,
					order = 14,
				},
				qualityThreshold = {
					name = L["Group Quality Threshold"],
					desc = L["Set the minimum quality of items looted by group members to show."],
					type = "select",
					values = qualityValues,
					order = 15,
				},
				display = {
					name = L["Item Display"],
					type = "group",
					inline = true,
					order = 20,
					args = {
						font = {
							name = L["Font"],
							desc = L["Set the font face."],
							type = "select",
							dialogControl = media and "LSM30_Font",
							values = media and media:HashTable("font") or {},
							set = function(info, value)
								db[info[#info]] = value
								UpdateFrameFont()
							end,
							order = 1,
							disabled = function() return not media end,
						},
						fontSize = {
							name = L["Font Size"],
							desc = L["Set the font size."],
							type = "range", min = 8, max = 30, step = 1,
							order = 2,
						},
						displayLength = {
							name = L["Item Text Length"],
							desc = L["Number of characters to show before truncating the item name."],
							type = "range", min = 1, softMax = 100, max = 300, step = 1,
							order = 3,
						},
						fadeDelay = {
							name = L["Fade Delay"],
							desc = L["Set the number of seconds to wait before fading the item."],
							type = "range", min = 0, max = 30, step = 1,
							order = 5,
						},
						fadeTime = {
							name = L["Fade Duration"],
							desc = L["Set the duration of the fade animation."],
							type = "range", min = 0, max = 1, step = .1,
							order = 6,
						},
						advancedDivider = {
							name = L["Advanced"],
							type = "header",
							order = 20,
						},
						showIcon = {
							name = L["Show Icon"],
							desc = L["Toggle showing the item icon."],
							type = "toggle",
							set = function(info, value)
								db[info[#info]] = value
								--invalidate frames, doesn't fix coin
								UpdateFrameBackdrop()
							end,
							order = 20.1,
						},
						iconSize = {
							name = L["Icon Size"],
							desc = L["Set the icon size."],
							type = "range", min = 8, max = 48, step = 1,
							order = 20.2,
							disabled = function() return not db.showIcon end,
						},
						rollIconScale = {
							name = L["Roll Icon Scale"],
							desc = L["Set the roll icon scale relative to the icon size."],
							type = "range", isPercent = true, min = 0, softMin = .1, softMax = 1, max = 3, step = .01,
							order = 20.3,
							disabled = function() return not db.showIcon end,
						},
						iconGap = {
							name = L["Text Offset"],
							desc = L["Set the spacing between the item icon and text."],
							type = "range", min = -100, softMin = -30, softMax = 30, max = 100, step = 1,
							order = 21,
						},
						rowSpacing = {
							name = L["Row Spacing"],
							desc = L["Set the spacing between rows."],
							type = "range", min = -100, softMin = -30, softMax = 30, max = 100, step = 1,
							set = function(info, value)
								db[info[#info]] = value
								UpdateRowOrdering()
								UpdateRows()
							end,
							order = 22,
						},
						strata = {
							name = L["Strata"],
							desc = L["Set the frame strata of items."],
							type = "select",
							values = strataValues,
							--why you sort my tables by key ;[
							get = function(info)
								local key = db[info[#info]]
								for i,v in ipairs(strataValues) do
									if v == key then
										return i
									end
								end
							end,
							set = function(info, value)
								db[info[#info]] = strataValues[value]
							end,
							order = 23,
						},
					},
				}, --display
				background = {
					name = L["Background"],
					type = "group",
					inline = true,
					order = 30,
					set = function(info, value)
						db[info[#info]] = value
						UpdateFrameBackdrop()
					end,
					disabled = function() return not media end,
					args = {
						bgFile = {
							name = L["Background"],
							desc = L["Set the background texture."],
							type = "select",
							dialogControl = media and "LSM30_Statusbar",
							values = media and media:HashTable("statusbar") or {},
							order = 1,
						},
						bgColor = {
							name = L["Backdrop Color"],
							desc = L["Set the backdrop color and opacity."],
							type = "color", hasAlpha = false,
							get = function(info)
								local color = db.bgColor
								return color.r, color.g, color.b, color.a
							end,
							set = function(info, r, g, b, a)
								local color = db.bgColor
								color.r, color.g, color.b, color.a = r, g, b, a
							end,
							order = 2,
						},
						colorBackground = {
							name = L["Color Background"],
							desc = L["Toggle tinting the background and border with the item quality color, otherwise, use the backdrop color."],
							type = "toggle",
							order = 2.1,
						},
						edgeFile = {
							name = L["Border"],
							desc = L["Set the border texture."],
							type = "select",
							dialogControl = media and "LSM30_Border",
							values = media and media:HashTable("border") or {},
							order = 3,
						},
						borderAlpha = {
							name = L["Border Alpha"],
							desc = L["Set the border's alpha value."],
							type = "range", isPercent = true, min = 0, max = 1, step = .05,
							order = 4,
							disabled = true,
							hidden = true,
						},
						edgeSize = {
							name = L["Border Size"],
							desc = L["Set the thickness of border segments and square size of the corners."],
							type = "range", min = 1, softMax = 16, max = 30, step = 1,
							order = 5,
						},
						insets = {
							name = L["Background Inset"],
							desc = L["Set distance from the edges of the frame to the edges of the background texture."],
							type = "range", min = 1, max = 16, step = 1,
							order = 6,
						},
					},
				}, --background
				sharedMediaMessage = {
					name = "** "..L["Some options are disabled because LibSharedMedia-3.0 is not available."],
					type = "description",
					order = 100,
					hidden = function() return media end,
				},
			},
		}
		return options
	end
end

function Touhin:OpenOptions()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
end


--------------------------------------------------------------------------------
-- Initialization
--

function Touhin:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("TouhinDB", defaults, true)

	local function OnProfileChanged()
		db = self.db.profile
		UpdateAnchor()
	end

	self.db.RegisterCallback(self, "OnProfileChanged", OnProfileChanged)
	self.db.RegisterCallback(self, "OnProfileCopied", OnProfileChanged)
	self.db.RegisterCallback(self, "OnProfileReset", OnProfileChanged)
	db = self.db.profile

	UpdateFrameBackdrop()
	UpdateFrameFont()

	LibStub("AceConfig-3.0"):RegisterOptionsTable("Touhin", GetOptions)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Touhin")
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Touhin Profile", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db))
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Touhin Profile", "Profiles", "Touhin")
end

function Touhin:OnEnable()
	if not db.anchor_point then
		ShowAnchor()
	end

	self:RegisterEvent("CHAT_MSG_LOOT")
	self:RegisterEvent("CHAT_MSG_MONEY")
	self:RegisterEvent("CHAT_MSG_CURRENCY")

	if db.disableLootAlert then
		AlertFrame:UnregisterEvent("LOOT_ITEM_ROLL_WON")
		AlertFrame:UnregisterEvent("SHOW_LOOT_TOAST")
	end

	--!ClassColors support
	if _G.CUSTOM_CLASS_COLORS then
		local function updateClassColors()
			for k,v in next, _G.CUSTOM_CLASS_COLORS do
				CLASS_COLORS[k] = string.format("|cff%02x%02x%02x", v.r * 255, v.g * 255, v.b * 255)
			end
		end
		_G.CUSTOM_CLASS_COLORS:RegisterCallback(updateClassColors)
		updateClassColors()
	else
		for k,v in next, _G.RAID_CLASS_COLORS do
			CLASS_COLORS[k] = string.format("|cff%02x%02x%02x", v.r * 255, v.g * 255, v.b * 255)
		end
	end
end

SLASH_TOUHIN1 = "/touhin"
SlashCmdList["TOUHIN"] = function(input)
	if input and input == "config" then
		Touhin:OpenOptions()
	else
		ToggleAnchor()
	end
end
