local sStats = CreateFrame("Frame", "sStats")
sStats.callbacks = sStats.callbacks or LibStub("CallbackHandler-1.0"):New(sStats)
local LSM = LibStub("LibSharedMedia-3.0")
local widgetLists = AceGUIWidgetLSMlists
local db, preHex, sufHex, colors
local _,class = UnitClass("player")
local modules = {}
local fontFlags = {"None", "Outline", "Monochrome Outline"}	--, "Monochrome"}
local alignments = {"RIGHT", "LEFT", "CENTER"}

local defaults = {
	font = "Arial Narrow",
	fontSize = 12,
	fontFlag = "None",
	align = "CENTER",
	locked = false,
	preColor = {r=1, g=1, b=1},
	sufColor = {r=1, g=1, b=1},
	classColor = false,
	shadow = true,
}


--in a separate function so it can be called by the options
local function ConfigModule(frame)
	local name = frame:GetName()
	local useGlobal = db[name].global
	
	frame:ClearAllPoints()
	if not db[name].point then
		db[name].point = {"CENTER", "UIParent", "CENTER", 0, 0}
	end
	frame:SetPoint(unpack(db[name].point))
	if not db.locked then
		frame:SetMovable(true)
		frame:SetBackdropColor(0,0,0,.4)
	else
		frame:SetMovable(false)
		frame:SetBackdropColor(0,0,0,0)
	end
	
	local fstring = _G[name.."Text"]
	fstring:ClearAllPoints()
	fstring:SetPoint(useGlobal and db.align or db[name].align)
	fstring:SetFont(LSM:Fetch("font", useGlobal and db.font or db[name].font), useGlobal and db.fontSize or db[name].fontSize, useGlobal and db.fontFlag or db[name].fontFlag)
	if (useGlobal and db.shadow) or (db[name].shadow and not useGlobal) then
		fstring:SetShadowColor(0,0,0,1)
	else
		fstring:SetShadowColor(0,0,0,0)
	end
end

--Frame and fonstring template setup--
function sStats:CreateModule(module)
	local f = CreateFrame("Frame", "sStats_"..module, UIParent)
		f:SetWidth(100)
		f:SetHeight(20)
		f:SetBackdrop({bgFile ="Interface\\Buttons\\WHITE8x8",})
		f:EnableMouse(true)
		f:SetScript("OnMouseDown", function(self) if self:IsMovable() then self:StartMoving() end end)
		f:SetScript("OnMouseUp", function(self)
					self:StopMovingOrSizing()
					local a,_,b,x,y = self:GetPoint()
					db[self:GetName()].point = {a, "UIParent", b,x,y}
					if sStats.optionsFrame[f:GetName()]:IsVisible() then
						sStats.optionsFrame[f:GetName()]:Hide()
						sStats.optionsFrame[f:GetName()]:Show()
					end
				end)
	local fstring = f:CreateFontString("sStats_"..module.."Text")
		fstring:SetShadowOffset(1,-1)

--	ConfigModule(f, fstring)
	tinsert(modules, f)
	
	return f, fstring
end

-- taken from User-Defined Functions pages - http://www.wowwiki.com/RGBPercToHex
local function RGBPercToHex(r,g,b)
	r = r <= 1 and r >= 0 and r or 0
	g = g <= 1 and g >= 0 and g or 0
	b = b <= 1 and b >= 0 and b or 0
	return string.format("%02x%02x%02x", r*255, g*255, b*255)
end


function sStats:SetModuleText(fstring, value, suffix)
	fstring:SetText("|cff"..preHex..value.."|r|cff"..sufHex..suffix)
end

local function SetUpDB()
	sStatsDB = sStatsDB or {}
	sStatsPCDB = sStatsPCDB or {}
		if sStatsPCDB.charSpec == nil then
			sStatsPCDB.charSpec = false
		end
	for k,v in pairs(defaults) do
	    if type(sStatsDB[k]) == "nil" then
	        sStatsDB[k] = v
	    end
	end
	if sStatsPCDB.charSpec then
		--set defaults if new charSpec DB
		for k,v in pairs(defaults) do
			if type(sStatsPCDB[k]) == "nil" then
				sStatsPCDB[k] = v
			end
		end
		db = sStatsPCDB
	else
		db = sStatsDB
	end

	local name
	for k,v in pairs(modules) do
		name = v:GetName()
		db[name] = db[name] or {}
		if db[name].global == nil then
			db[name].global = true
		end
		db[name].font = db[name].font or db.font
		db[name].fontFlag = db[name].fontFlag or db.fontFlag
		db[name].fontSize = db[name].fontSize or db.fontSize
		db[name].align = db[name].align or db.align
		if db[name].shadow == nil then
			db[name].shadow = true
		end
	end
end


sStatsOptionsTable = {
	name = "sStats",
	type = "group",
	args = {
		general = {
			name = "Global",
			type = "group",
			args = {
				locked = {
					name = "Lock Frames",
					desc = "Lock the module frames, hiding the background and preventing them from being moved.",
					type = "toggle",
					get = function() return db.locked end,
					set = function()
							db.locked = not db.locked
							if db.locked then
								for k,v in pairs(modules) do
									v:SetMovable(false)
									v:SetBackdropColor(0,0,0,0)
								end
							else
								for k,v in pairs(modules) do
									v:SetMovable(true)
									v:SetBackdropColor(0,0,0,.4)
								end
							end
						end,
					order = 0,
				},
				header1 = {
					name = "Global Options",
					type = "header",
					order = 1,
				},
				font = {
					name = "Font",
					desc = "Select the font for all modules to display.",
					type = "select",
					dialogControl = "LSM30_Font",
					values = widgetLists.font,
					get = function()
							return db.font
						end,
					set = function(self, font)
							db.font = font
							for k,v in pairs(modules) do
								ConfigModule(v)
							end
						end,
					order = 2,
				},
				fontFlag = {
					name = "Font Flag",
					desc = "Set how to alter the modules' font.",
					type = "select",
					values = fontFlags,
					get = function()
								for k, v in pairs(fontFlags) do
									if db.fontFlag == v then
										return k
									end
								end
							end,
					set = function(_,key)
								db.fontFlag = fontFlags[key]
								for k,v in pairs(modules) do
									ConfigModule(v)
								end
							end,
					order = 3,
				},
				fontSize = {
					name = "Font Size",
					desc = "Change the font size of the modules.",
					type = "range",
					min = 6,
					max = 24,
					step = 1,
					get = function() return db.fontSize end,
					set = function(_,size) 
								db.fontSize = size
								for k,v in pairs(modules) do
									ConfigModule(v)
								end
							end,
					order = 4,
				},
				align = {
					name = "Font Alignment",
					desc = "What side of the module's frame you would like the text aligned to.",
					type = "select",
					values = alignments,
					get = function()
								for k, v in pairs(alignments) do
									if db.align == v then
										return k
									end
								end
							end,
					set = function(_, align)
								db.align = alignments[align]
								for k,v in pairs(modules) do
									ConfigModule(v)
								end
							end,
					order = 5,
				},
				preColor = {
					name = "Prefix Text Color",
					desc = "The color of the module's value.",
					type = "color",
					get = function() return db.preColor.r, db.preColor.g, db.preColor.b end,
					set = function(_,r,g,b)
							db.preColor.r = r
							db.preColor.g = g
							db.preColor.b = b
							preHex = RGBPercToHex(r,g,b)
						end,
					order = 6,
				},
				sufColor = {
					name = "Suffix Text Color",
					desc = "The color of the module's suffix text.",
					type = "color",
					disabled = function() return db.classColor end,
					get = function() return db.sufColor.r, db.sufColor.g, db.sufColor.b end,
					set = function(_,r,g,b)
							db.sufColor.r = r
							db.sufColor.g = g
							db.sufColor.b = b
							sufHex = RGBPercToHex(r,g,b)
						end,
					order = 7,
				},
				classColor = {
					name = "Class colored suffix",
					desc = "Color the module's suffix text by class.",
					type = "toggle",
					get = function() return db.classColor end,
					set = function()
							db.classColor = not db.classColor
							if db.classColor then
								sufHex = RGBPercToHex(colors[class].r,colors[class].g,colors[class].b)
							else
								sufHex = RGBPercToHex(db.sufColor.r,db.sufColor.g,db.sufColor.b)
							end
						end,
					order = 8,
				},
				shadow = {
					name = "Font Shadow",
					desc = "Give the text a shadow.",
					type = "toggle",
					get = function() return db.shadow end,
					set = function() 
								db.shadow = not db.shadow
								for k,v in pairs(modules) do
									ConfigModule(v)
								end
							end,
					order = 9,
				},
			},
		},
		profile = {
			name = "Profile",
			type = "group",
			args = {
				header7 = {
					name = "",
					type = "header",
					order = 1,
				},
				charSpec = {
					name = "Character specific settings",
					desc = "Have this character use their own profile.  If checked, any changes made will not affect other characters.\n\n|c00E30016WARNING:|r Your UI will be reloaded in the process!",
					type = "toggle",
					width = "full",
					confirm = true,
					get = function() return sStatsPCDB.charSpec end,
					set = function()
								sStatsPCDB.charSpec = not sStatsPCDB.charSpec
								ReloadUI()
							end,
					order = 2,
				},
				copyProfile = {
					name = "Copy from Default",
					desc = "Copy all settings from the default, global profile to this character's profile.  This will not effect other characters' specific profiles.\n\n|c00E30016WARNING:|r Your UI will be reloaded in the process!",
					type = "execute",
					confirm = true,
					disabled = function() return not sStatsPCDB.charSpec end,
					func = function()
								sStatsPCDB = sStatsDB
								sStatsPCDB.charSpec = true
								ReloadUI()
							end,
					order = 3,
				},
				resetProfile = {
					name = "Profile Reset",
					desc = "Reset this profile back to the out-of-the-box settings.  If you reset the character specific profile, the global profile will be untouched, and vice versa.  This will not effect other character specific profiles.\n\n|c00E30016WARNING:|r Your UI will be reloaded in the process!",
					type = "execute",
					confirm = true,
					func = function()
								if sStatsPCDB.charSpec then
									sStatsPCDB = {charSpec = true}
								else
									sStatsDB = {}
								end
								ReloadUI()
							end,
					order = 4,
				},
			},
		},
	},
}

local options = sStatsOptionsTable
local function GetModuleOptions(module)
	options.args[module] = {
		name = module,
		type = "group",
		args = {
			header = {
				name = module.." Options",
				type = "header",
				order = 0,
			},
			Xoffset = {
				name = "X:",
				desc = "The module's X coordinate on the screen.",
				type = "input",
				get = function() return tostring(db[module].point[4]) end,
				set = function(_,x)
							if x == nil or x == "" then
								x = "0"
							end
							db[module].point[4] = x
							_G[module]:ClearAllPoints()
							_G[module]:SetPoint(unpack(db[module].point))
						end,
				order = 1,
			},
			Yoffset = {
				name = "Y:",
				desc = "The module's Y coordinate on the screen.",
				type = "input",
				get = function() return tostring(db[module].point[5]) end,
				set = function(_,y)
							if y == nil or y == "" then
								y = "0"
							end
							db[module].point[5] = y
							_G[module]:ClearAllPoints()
							_G[module]:SetPoint(unpack(db[module].point))
						end,
				order = 2,
			},
			global = {
				name = "Use Global Settings",
				desc = "Use the global settings for this module.",
				type = "toggle",
				width = "full",
				get = function() return db[module].global end,
				set = function() db[module].global = not db[module].global end,
				order = 3,
			},
			custom = {
				name = "Custom Module Settings",
				type = "group",
				inline = true,
				order = 4,
				disabled = function() return db[module].global end,
				args = {
					font = {
						name = "Font",
						desc = "Select the font for this module to display.",
						type = "select",
						dialogControl = "LSM30_Font",
						values = widgetLists.font,
						get = function()
								return db[module].font
							end,
						set = function(self, font)
								db[module].font = font
								ConfigModule(_G[module])
							end,
						order = 1,
					},
					fontFlag = {
						name = "Font Flag",
						desc = "Set how to alter the modules' font.",
						type = "select",
						values = fontFlags,
						get = function()
									for k, v in pairs(fontFlags) do
										if db[module].fontFlag == v then
											return k
										end
									end
								end,
						set = function(_,key)
									db[module].fontFlag = fontFlags[key]
									ConfigModule(_G[module])
								end,
						order = 2,
					},
					fontSize = {
						name = "Font Size",
						desc = "Change the font size of this module.",
						type = "range",
						min = 6,
						max = 24,
						step = 1,
						get = function() return db[module].fontSize end,
						set = function(_,size) 
									db[module].fontSize = size
									ConfigModule(_G[module])
								end,
						order = 3,
					},
					align = {
						name = "Font Alignment",
						desc = "What side of this module's frame you would like the text aligned to.",
						type = "select",
						values = alignments,
						get = function()
									for k, v in pairs(alignments) do
										if db[module].align == v then
											return k
										end
									end
								end,
						set = function(_, align)
									db[module].align = alignments[align]
									ConfigModule(_G[module])
								end,
						order = 4,
					},
					shadow = {
						name = "Font Shadow",
						desc = "Give this module's text a shadow.",
						type = "toggle",
						get = function() return db[module].shadow end,
						set = function() 
									db[module].shadow = not db[module].shadow
									ConfigModule(_G[module])
								end,
						order = 5,
					},
				},
			},
		},
	}
	sStats.optionsFrame[module] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("sStats", module, "sStats", module)
end


local function OnInitialize()
	SetUpDB()
		
	LibStub("AceConfig-3.0"):RegisterOptionsTable("sStats", options)
	sStats.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("sStats", "sStats", nil, "general")
	for k,v in pairs(modules) do
		GetModuleOptions(v:GetName())
	end
	sStats.optionsFrame.profile = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("sStats", "Profile", "sStats", "profile")

	colors = CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS	--support for CUSTOM_CLASS_COLORS addons
	if CUSTOM_CLASS_COLORS then
		CUSTOM_CLASS_COLORS:RegisterCallback(function()
				colors = CUSTOM_CLASS_COLORS
				if db.classColor then
					sufHex = RGBPercToHex(colors[class].r,colors[class].g,colors[class].b)
				end
			end)
	end
	
	preHex = RGBPercToHex(db.preColor.r,db.preColor.g,db.preColor.b)
	if db.classColor then
		sufHex = RGBPercToHex(colors[class].r,colors[class].g,colors[class].b)
	else
		sufHex = RGBPercToHex(db.sufColor.r,db.sufColor.g,db.sufColor.b)
	end
	
	for k,v in pairs(modules) do
		ConfigModule(v)
	end
	sStats.callbacks:Fire("sStats_Modules_Ready", "Fired")
	
	
	SlashCmdList["SSTATS"] = function() InterfaceOptionsFrame_OpenToCategory("sStats") end
	SLASH_SSTATS1 = "/sstats"
end

--sStats:RegisterEvent("ADDON_LOADED")
sStats:RegisterEvent("VARIABLES_LOADED")
sStats:SetScript("OnEvent", function(self,event,addon)
		--if addon == "sStats" then
			OnInitialize()
		--end
	end)