local ShowPercent = true
local ShowCurrent = false	-- Takes priority over ShowRemaining
local ShowRemaining = false	-- Won't show if ShowCurrent = true

local sStats_exp, sStats_expText = sStats:CreateModule("exp")

local function FormatToDecimalPlaces(num, places)
	local placeValue = ("%%.%df"):format(places)
	return placeValue:format(num)
end
local function ReadableNumber(num)
	local ret = 0
	if not num then
		return 0
	elseif num >= 100000000 then
		ret = FormatToDecimalPlaces(num / 1000000, 0) .. "m" -- hundred million
	elseif num >= 10000000 then
		ret = FormatToDecimalPlaces(num / 1000000, 1) .. "m" -- ten million
	elseif num >= 1000000 then
		ret = FormatToDecimalPlaces(num / 1000000, 2) .. "m" -- million
	elseif num >= 100000 then
		ret = FormatToDecimalPlaces(num / 1000, 0) .. "k" -- hundred thousand
	elseif num >= 10000 then
		ret = FormatToDecimalPlaces(num / 1000, 1) .. "k" -- ten thousand
	elseif num >= 1000 then
		ret = FormatToDecimalPlaces(num / 1000, 2) .. "k" -- thousand
	else
		ret = num -- hundreds
	end
	return ret
end

local function UpdateExp()
	local CurXP = UnitXP("player")
	local XPMax = UnitXPMax("player")
	local CurLvl = UnitLevel("player")
	local RestXP = GetXPExhaustion()
	
	local PerXP = (CurXP/XPMax) * 100
	
	local PerXPString = format("%.1f%%", PerXP)
	local RestXPString = format("%.1f%%", (RestXP/XPMax) * 100)

	-- Tooltip
	sStats_exp:SetScript("OnEnter", function(this)
		GameTooltip:SetOwner(this, "ANCHOR_CURSOR")
		GameTooltip:AddLine("|cffffd200Experience|r")
		GameTooltip:AddDoubleLine("Current", ReadableNumber(CurXP), 1, 1, 1, 1, 1, 1)
		GameTooltip:AddDoubleLine("Remaining", ReadableNumber(XPMax - CurXP), 1, 1, 1, 1, 1, 1)
		
		if RestXP == nil then
			GameTooltip:AddDoubleLine("Rested", "0", 1, 1, 1, 1, 1, 1)
		else
			GameTooltip:AddDoubleLine("Rested", ReadableNumber(RestXP), 1, 1, 1, 1, 1, 1)
		end

		GameTooltip:Show()
	end)
	sStats_exp:SetScript("OnLeave", function()
		if GameTooltip:IsShown() then
		GameTooltip:Hide()
		end
	end)
	
	-- Text
	local displayString = ""
	if ShowCurrent then
		displayString = ReadableNumber(CurXP)
	elseif ShowRemaining then
		displayString = ReadableNumber(XPMax - CurXP)
	end
	if ShowPercent then
		displayString = displayString .. ((ShowCurrent or ShowRemaining) and " - " or "") .. PerXPString
	end
	if RestXP > 0 then
		sStats:SetModuleText(sStats_expText, displayString.." (R: "..RestXPString..")", "")
	else
		sStats:SetModuleText(sStats_expText, displayString, "")
	end
end

sStats.RegisterCallback(sStats_exp, "sStats_Modules_Ready", function()

	sStats_exp:RegisterEvent("PLAYER_ENTERING_WORLD")
	sStats_exp:RegisterEvent("PLAYER_XP_UPDATE")

	sStats_exp:SetScript("OnEvent", function()			
		UpdateExp()		
	end)

end)